@@ -6,8 +6,9 @@ import os
 import numpy as np
 from astropy import units as u
 from ctapipe.calib.camera.flatfield import FlatFieldCalculator
-from ctapipe.core.traits import  List, Unicode
+from ctapipe.core.traits import  List, Unicode, Bool
 from lstchain.calib.camera.pulse_time_correction import PulseTimeCorrection
+from lstchain.calib.camera.calibrator import get_charge_correction
 
 __all__ = [
     'FlasherFlatFieldCalculator'
@@ -92,7 +93,6 @@ class FlasherFlatFieldCalculator(FlatFieldCalculator):
                 raise IOError(msg)
 
 
-
     def _extract_charge(self, event):
         """
         Extract the charge and the time from a calibration event
