@@ -243,6 +243,8 @@ def r0_to_dl1(
                                                 time_calibration_path = time_calibration_path,
                                                 extractor_product = config['image_extractor'],
                                                 gain_threshold = Config(config).gain_selector_config['threshold'],
+                                                charge_scale = config['charge_scale'],
+                                                apply_charge_correction = Config(config).LSTCalibrationCalculator.apply_charge_correction,
                                                 config = Config(config),
                                                 allowed_tels = [1],
                                                 )
@@ -253,6 +255,8 @@ def r0_to_dl1(
                                                                time_calibration_path = time_calibration_path,
                                                                extractor_product = config['image_extractor_for_muons'],
                                                                gain_threshold = Config(config).gain_selector_config['threshold'],
+                                                               charge_scale=config['charge_scale'],
+                                                               apply_charge_correction=Config(config).LSTCalibrationCalculator.apply_charge_correction,
                                                                config = Config(config),
                                                                allowed_tels = [1],)
 
@@ -316,7 +320,7 @@ def r0_to_dl1(
             # the final transform then needs the mapping and the number of telescopes
             tel_list_transform = partial(
                 utils.expand_tel_list,
-                max_tels=len(first_event.inst.subarray.tel) + 1,
+                max_tels=max(first_event.inst.subarray.tel) + 1,
             )
 
             writer.add_column_transform(
@@ -348,6 +352,8 @@ def r0_to_dl1(
                 write_subarray_tables(writer, event, metadata)
                 if not custom_calibration:
                     cal_mc(event)
+                if config['mc_image_scaling_factor'] != 1:
+                    rescale_dl1_charge(event, config['mc_image_scaling_factor'])
 
             else:
                 if i==0:
@@ -719,3 +725,18 @@ def add_disp_to_parameters_table(dl1_file, table_path, focal):
         add_column_table(tab, tables.Float32Col, 'src_x', source_pos_in_camera.x.value)
         tab = file.root[table_path]
         add_column_table(tab, tables.Float32Col, 'src_y', source_pos_in_camera.y.value)
+
+
+def rescale_dl1_charge(event, scaling_factor):
+    """
+    Rescale the charges (images) by a given scaling factor.
+    The images in dl1.tel[tel_id].image is directly multiplied in place by `scaling_factor`.
+
+    Parameters
+    ----------
+    event: `ctapipe.io.containers.DataContainer`
+    scaling_factor: float
+    """
+
+    for tel_id, tel in event.dl1.tel.items():
+        tel.image *= scaling_factor
\ No newline at end of file
