@@ -1,20 +1,37 @@
 from lstchain.reco import utils
 import astropy.units as u
 from astropy.time import Time
+from astropy.coordinates import SkyCoord
 import numpy as np
 import pandas as pd
 
 
-def test_camera_to_sky():
+def test_camera_to_altaz():
     pos_x = np.array([0, 0]) * u.m
     pos_y = np.array([0, 0]) * u.m
     focal = 28*u.m
     pointing_alt = np.array([1.0, 1.0]) * u.rad
     pointing_az = np.array([0.2, 0.5]) * u.rad
-    sky_coords = utils.camera_to_sky(pos_x, pos_y, focal, pointing_alt, pointing_az)
+    sky_coords = utils.camera_to_altaz(pos_x, pos_y, focal, pointing_alt, pointing_az)
     np.testing.assert_allclose(sky_coords.alt, pointing_alt, rtol=1e-4)
     np.testing.assert_allclose(sky_coords.az, pointing_az, rtol=1e-4)
 
+    # Test for real event with a time
+    obs_time = Time('2018-11-01T02:00', '2018-11-01T02:00')
+    sky_coords = utils.camera_to_altaz(pos_x, pos_y, focal, pointing_alt, pointing_az, obstime = obs_time)
+    np.testing.assert_allclose(sky_coords.alt, pointing_alt, rtol=1e-4)
+    np.testing.assert_allclose(sky_coords.az, pointing_az, rtol=1e-4)
+
+def test_radec_to_camera():
+    pointing_radec = SkyCoord.from_name('Crab')
+    obstime = Time('2020-01-27T23:00', scale='utc')
+    pointing_alt  = u.Quantity(1.3748, u.rad, copy=False)
+    pointing_az = u.Quantity(4.0975, u.rad, copy=False)
+    focal = 28*u.m
+    expected_source_pos_camera = np.array([0.0, 0.0]) * u.m
+    pointing_pos_camera = utils.radec_to_camera(pointing_radec, obstime, pointing_alt, pointing_az, focal)
+    np.testing.assert_allclose(pointing_pos_camera.x.to_value(), expected_source_pos_camera[0].to_value(), atol=0.1)
+    np.testing.assert_allclose(pointing_pos_camera.y.to_value(), expected_source_pos_camera[1].to_value(), atol=0.1)
 
 def test_reco_source_position_sky():
     cog_x = np.array([2, 1]) * u.m
